#!/usr/bin/perl
#
# (c) astx (at) aws-it.at, 2018
#
# $Id: generate_rms_power_dissipation_script.pl 1562 2019-01-06 12:04:33Z sta $
#

use strict;
my $s=0;
my $devs={};
my @lines;

if (! -f "$ARGV[0]") {
	print STDERR "input file <$ARGV[0]> does not exists.\nYou need to call me so: $0 inputfile outputfile\n";
}

open(INP, "< $ARGV[0]") && do {
	while (my $line=<INP>) {
		chomp($line);
		# *==============  Begin SPICE netlist of main design 
		if ($line =~ /^.*Begin SPICE netlist of main design.*/) {
			$s=1;
		}
		if ($s) {
			# known devices:
			my $d=lc($line);
			if ($d=~/^([rdclmq][a-z]*)([0-9]+)\s+/) {
				my $dtyp="$1";
				my $num="$2";
				$devs->{"$dtyp"}{"$num"}=$d;
			}
		}
	}
	close(INP);
};

open(OUT, "> $ARGV[1]") && do {

	foreach my $dtyp (sort keys %{$devs}) {
		foreach my $num (sort {$a<=>$b} keys %{$devs->{$dtyp}}) {
			push(@lines, "$devs->{$dtyp}{$num}");
		}
	}

	print OUT "\n.control\n";
	foreach my $line (@lines) {
		my $d=lc($line);
		if ($d=~/^([rdcl][^\s]+)\s+([^\s]+)\s+([^\s]+)\s+/) {
			my $dev="$1";
			my $n1="$2";
			my $n2="$3";
			# DEV r2 connected via pgnd and 0
			if (defined $ENV->{'MDEBUG'}) { print STDERR "DEV $dev connected via $n1 and $n2\n"; }
			my $c='i';
			# diode has other current var name
			if ($dev=~/^d/) {
				$c='id';
			}
			if ($n1 eq '0' || $n2 eq '0') {
				if ($n1 eq '0') {
					# add c-e / d-s voltage too for SOA plots
					print OUT "let v_$dev = abs(v($n2))\n";
				} else {
					# add c-e / d-s voltage too for SOA plots
					print OUT "let v_$dev = abs(v($n1))\n";
				}
			} else {
				# add c-e / d-s voltage too for SOA plots
				print OUT "let v_$dev = abs(v($n1,$n2))\n";
			}

			print OUT "let i_$dev = abs(\@$dev\[$c\])\n";
			print OUT "let p_$dev = i_$dev*v_$dev\n";
			print OUT "meas tran pwr_rms_$dev RMS p_$dev\n";

			print OUT "settype voltage v_$dev\n";
			print OUT "settype power p_$dev\n";
			print OUT "settype power pwr_rms_$dev\n";

			print OUT "let \{\$c\}.pwr_$dev\[runs\]=pwr_rms_$dev\n";

		}
		if ($d=~/^([mq][^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+/) {
			my $dev=$1;
			my $n1=$2;
			my $n2=$3;
			my $n3=$4;

			# MQ22 railpos 29 vpfet1 IXTH80N20L => D G S
			# MQ22 railpos 29 vpfet1 IXTH80N20L 
			# MQ24 railneg 28 vnfet1 IXTH48P20P 
			# Q21 lrailpos 47 vemp1 2SC5200_k 
			# Q23 lrailneg 48 vemn1 2SA1943_k 
			my $c='ie';
			# mosfet has other current var name
			if ($dev=~/^m/) {
				$c='id';
			}
			if ($n1 eq '0' || $n3 eq '0') {
				if ($n1 eq '0') {
					# add c-e / d-s voltage too for SOA plots
					print OUT "let v_$dev = abs(v($n3))\n";
				} else {
					# add c-e / d-s voltage too for SOA plots
					print OUT "let v_$dev = abs(v($n1))\n";
				}
			} else {
				# add c-e / d-s voltage too for SOA plots
				print OUT "let v_$dev = abs(v($n1,$n3))\n";
			}

			print OUT "let i_$dev = abs(\@$dev\[$c\])\n";
			print OUT "let p_$dev = i_$dev*v_$dev\n";
			print OUT "meas tran pwr_rms_$dev RMS p_$dev\n";

			print OUT "settype voltage v_$dev\n";
			print OUT "settype power p_$dev\n";
			print OUT "settype power pwr_rms_$dev\n";

			print OUT "let \{\$c\}.pwr_$dev\[runs\]=pwr_rms_$dev\n";
			if (defined $ENV->{'MDEBUG'}) { print STDERR "DEV $dev connected via D=$n1 and S=$n3 and G=$n2\n"; }
		}
	}

	print OUT "let \{\$c\}.pwr_steps\[runs\]=runs\n";

	print OUT "\n.endc\n\n";
	close(OUT);
};
